/*
Lampeggio led su interrupt del Timer A
e controllo di un pulsante
Target: MSP430G2221 su Launchpad
Compilatore: CCS

Autore: Bernardo Giovanni (gianni@settorezero.com)
www.settorezero.com

Tutti gli articoli sugli MSP430:
http://www.settorezero.com/wordpress/category/microcontrollori/msp430/

E' FATTO DIVIETO ASSOLUTO DI RIDISTRIBUIRE QUESTO CODICE
O PARTI DI ESSO SENZA L'ESPLICITA AUTORIZZAZIONE DELL'AUTORE

RISPETTATE LE VOLONTA' DEGLI AUTORI E RISPETTATE I COPYRIGHT!
*/


// Header generico: l'MSP430 giusto viene selezionato
// in base alla MCU scelta in fase di creazione del progetto
#include "msp430.h"

// Macro per i led
// LED1 (rosso) su P1.0
#define LED1ON	(P1OUT | BIT0)
#define LED1OFF	(P1OUT & !BIT0)
#define LED1TOGGLE (P1OUT ^= BIT0)
// LED2 (verde) su P1,6
#define LED2ON	(P1OUT | BIT6)
#define LED2OFF	(P1OUT & !BIT6)
#define LED2TOGGLE (P1OUT ^= BIT6)
// Macro per rilevare la pressione del pulsante S2
// Il pulsante S2 tira P1.3 verso massa
// per cui dopo dobbiamo attivare la resistenza di pullup su P1.3
#define S2PUSH !(P1IN & BIT3)

// contatore per lampeggio
unsigned int timerCount = 8;
 
int main(void)
	{
	WDTCTL = WDTPW + WDTHOLD; // disabilito il Watchdog timer
 	P1DIR = 0x41; // P1.6 e P1.0 come uscite
  	
 	P1REN |= BIT3; // Resistenza attivata su P1.3
 	P1OUT |= BIT3; // bit a 1 = La resistenza attivata  quella di pullup
  
 	// Setup Timer A
 	// TACTL  il registro di controllo del Timer_
 	// TASSEL_2 => Sorgente di clock derivata da SMCLCK (bits 9,8)
 	// MC_2 => Modo continuo: il timer conta da 0 a 0xFF, scatta l'interrupt e si azzera (bits 5,4)
 	// TAIE => Interrupt abilitato
 	TACTL = TASSEL_2 + MC_2 + TAIE;
    
 	// Abilita gli interrupt,  una macro intrinseca
 	__enable_interrupt();

	// Led spenti all'avvio
 	LED1OFF;
 	LED2OFF;
  
 	while(1)
		{
		// controllo pressione pulsante
		if (S2PUSH)
			{
			__delay_cycles (70000); // ritardo di 70mila cicli, circa 70mS con clock interno a 1MHz
  			if (S2PUSH)
  				{
  				LED1TOGGLE; // inverto lo stato del led
  				}
  			while(S2PUSH); // blocco l'esecuzione del programma fino a quando il pulsante non viene rilasciato
  			}
  		}// \while(1)
	}// \main

// Timer A0 interrupt service routine
// fare riferimento al paragrafo
// 5.9.16 The vector Pragma, pag.91
// documento SLAU132E.pdf
#pragma vector=TIMERA1_VECTOR
// TIMERA0_VECTOR : indirizzo 0xFFF2 => Timer A CC0
// TIMERA1_VECTOR : indirizzo 0xFFF0 => Timer A CC1, TAIFG
// la tabella interrupt si trova a pag.9 del datasheet
// il nome mnemnoico TIMERA0_VECTOR si trova nel file header dell'MSP430
__interrupt void timer_a_interrupt (void)
	{
	// Controlliamo che il salto a questo vettore sia stato causato
	// dall'interrupt per overflow.
	// Il bit TAIFG non  definito direttamente ma come maschera
	if (TACTL & TAIFG)
		{
		timerCount--; // decremento il contatore
		if(timerCount==0)
			{
			LED2TOGGLE; // inverto lo stato del led
			timerCount=8; // ricarico il contatore
			}
		}
 	// azzero tutti i flag di interrupt associati al timer A
	TAIV=0;
	}
	